<?php

namespace Drupal\migrate_boost\Decorators;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate_boost\MigrateBoost;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * ModuleHandler decorator.
 */
class MigrateBoostModuleHandler implements ModuleHandlerInterface, ContainerInjectionInterface {

  /**
   * Configuration for migrate_boost settings.
   */
  protected ?ImmutableConfig $config = NULL;

  public function __construct(
    protected readonly ModuleHandlerInterface $moduleHandler,
    protected readonly ContainerInterface $container,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('module_handler'),
      $container
    );
  }

  /**
   * Lazy load the migrate_boost settings configuration.
   */
  protected function getConfig($name): array {
    if (!$this->config) {
      $this->config = $this->container->get('config.factory')->get('migrate_boost.settings');
    }
    return $this->config->get($name) ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function load($name) {
    return $this->moduleHandler->load($name);
  }

  /**
   * {@inheritdoc}
   */
  public function loadAll() {
    $this->moduleHandler->loadAll();
  }

  /**
   * {@inheritdoc}
   */
  public function isLoaded() {
    return $this->moduleHandler->isLoaded();
  }

  /**
   * {@inheritdoc}
   */
  public function reload() {
    $this->moduleHandler->reload();
  }

  /**
   * {@inheritdoc}
   */
  public function getModuleList() {
    return $this->moduleHandler->getModuleList();
  }

  /**
   * {@inheritdoc}
   */
  public function getModule($name) {
    return $this->moduleHandler->getModule($name);
  }

  /**
   * {@inheritdoc}
   */
  public function setModuleList(array $module_list = []) {
    $this->moduleHandler->setModuleList($module_list);
  }

  /**
   * {@inheritdoc}
   */
  public function addModule($name, $path) {
    $this->moduleHandler->addModule($name, $path);
  }

  /**
   * {@inheritdoc}
   */
  public function addProfile($name, $path) {
    $this->moduleHandler->addProfile($name, $path);
  }

  /**
   * {@inheritdoc}
   */
  public function buildModuleDependencies(array $modules) {
    return $this->moduleHandler->buildModuleDependencies($modules);
  }

  /**
   * {@inheritdoc}
   */
  public function moduleExists($module) {
    return $this->moduleHandler->moduleExists($module);
  }

  /**
   * {@inheritdoc}
   */
  public function loadAllIncludes($type, $name = NULL) {
    return $this->moduleHandler->loadAllIncludes($type, $name);
  }

  /**
   * {@inheritdoc}
   */
  public function loadInclude($module, $type, $name = NULL) {
    return $this->moduleHandler->loadInclude($module, $type, $name);
  }

  /**
   * {@inheritdoc}
   */
  public function getHookInfo() {
    return $this->moduleHandler->getHookInfo();
  }

  /**
   * {@inheritdoc}
   */
  public function writeCache() {
    $this->moduleHandler->writeCache();
  }

  /**
   * {@inheritdoc}
   */
  public function resetImplementations() {
    $this->moduleHandler->resetImplementations();
  }

  /**
   * {@inheritdoc}
   */
  public function getModuleDirectories() {
    return $this->moduleHandler->getModuleDirectories();
  }

  /**
   * {@inheritdoc}
   */
  public function getName($module) {
    return $this->moduleHandler->getName($module);
  }

  /**
   * {@inheritdoc}
   */
  public function hasImplementations(string $hook, $modules = NULL): bool {
    $skip_modules_all_hooks = $this->getModulesAllHooksDisabled();
    $modules = array_diff((array) $modules, $skip_modules_all_hooks);
    $filtered_modules = array_filter($modules, fn($module) => !$this->isHookDisabled($hook, $module));

    return $this->moduleHandler->hasImplementations($hook, $filtered_modules);
  }

  /**
   * {@inheritdoc}
   */
  public function invokeAllWith(string $hook, callable $callback): void {
    $skip_modules_for_hook = $this->getDisabledModulesForHook($hook);
    $skip_modules_all_hooks = $this->getModulesAllHooksDisabled();
    $skip_modules = array_merge($skip_modules_for_hook, $skip_modules_all_hooks);
    $this->moduleHandler->invokeAllWith($hook, fn ($listener, $module) => in_array($module, $skip_modules) ?: $callback($listener, $module));
  }

  /**
   * {@inheritdoc}
   */
  public function invoke($module, $hook, array $args = []) {
    if ($this->isHookDisabled($hook, $module) || in_array($module, $this->getModulesAllHooksDisabled(), TRUE)) {
      return NULL;
    }
    return $this->moduleHandler->invoke($module, $hook, $args);
  }

  /**
   * {@inheritdoc}
   */
  public function invokeAll($hook, array $args = []) {
    $return = [];
    $this->invokeAllWith($hook, function (callable $callable_hook, string $module) use ($args, &$return, $hook) {
      if ($this->isHookDisabled($hook, $module)) {
        return;
      }
      if (in_array($module, $this->getModulesAllHooksDisabled())) {
        return;
      }
      $result = call_user_func_array($callable_hook, $args);
      if (isset($result) && is_array($result)) {
        $return = NestedArray::mergeDeep($return, $result);
      }
      elseif (isset($result)) {
        $return[] = $result;
      }
    });

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function invokeDeprecated($description, $module, $hook, array $args = []) {
    return $this->isHookDisabled($hook, $module) ? NULL : $this->moduleHandler->invokeDeprecated($description, $module, $hook, $args);
  }

  /**
   * {@inheritdoc}
   */
  public function invokeAllDeprecated($description, $hook, array $args = []) {
    if (!$this->isHookDisabled($hook)) {
      return $this->moduleHandler->invokeAllDeprecated($description, $hook, $args);
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function alter($type, &$data, &$context1 = NULL, &$context2 = NULL) {
    $this->moduleHandler->alter($type, $data, $context1, $context2);
  }

  /**
   * {@inheritdoc}
   */
  public function alterDeprecated($description, $type, &$data, &$context1 = NULL, &$context2 = NULL) {
    $this->moduleHandler->alterDeprecated($description, $type, $data, $context1, $context2);
  }

  /**
   * {@inheritdoc}
   */
  public function destruct() {
    $this->moduleHandler->destruct();
  }

  /**
   * Checks if a hook is disabled for a module.
   */
  protected function isHookDisabled(string $hook, ?string $module = NULL): bool {
    if (!MigrateBoost::isBoostActive()) {
      return FALSE;
    }

    // Hook is disabled for module.
    $hooks = $this->getConfig('hooks')[$hook] ?? [];

    return in_array($module, $hooks, TRUE);
  }

  /**
   * Returns a list of modules that have all hooks disabled.
   */
  protected function getModulesAllHooksDisabled(): array {
    if (!MigrateBoost::isBoostActive()) {
      return [];
    }

    return $this->getConfig('modules');
  }

  /**
   * Returns a list of modules that a hook is disabled for.
   */
  protected function getDisabledModulesForHook(string $hook): array {
    if (!MigrateBoost::isBoostActive()) {
      return [];
    }

    // All modules the hook is disabled for.
    return $this->getConfig('hooks')[$hook] ?? [];
  }

}
